#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#ifdef LB_ENABLE_MPI
#include <mpi.h>
#include "lb-mpi-utils.h"
#endif /* LB_ENABLE_MPI */

#include <lualib.h>

#include "lb-messages.h"

#include "lblua-main.h"

#include "lua-lb-misc.h"
#include "lua-lb-d2q9-VdW.h"
#include "lua-lb-d3q19-VdW.h"

#ifdef LB_ENABLE_RGB
#include "lua-lb-rgb.h"
#endif /* LB_ENABLE_RGB */

#ifndef LUA_LB_MODULENAME
#define LUA_LB_MODULENAME "lb"
#endif /* LUA_LB_MODULENAME */

static int
luaopen_lb(lua_State* ls)
{
	lb_assert(ls != NULL);

	lua_lb_misc_register(ls, LUA_LB_MODULENAME);
	lua_lb_d2q9_VdW_register(ls, LUA_LB_MODULENAME);
	lua_lb_d3q19_VdW_register(ls, LUA_LB_MODULENAME);

#ifdef LB_ENABLE_RGB
	lua_lb_rgb_register(ls, LUA_LB_MODULENAME);
#endif /* LB_ENABLE_RGB */

	return 0;
}

int
main(int argc, char** argv)
{
	const lua_CFunction modules[] = {
		luaopen_base, luaopen_table,
		luaopen_io, luaopen_string,
		luaopen_math, luaopen_debug,
		luaopen_lb, NULL
	};

#ifdef LB_ENABLE_MPI
	LB_CALL_MPI(MPI_Init(&argc, &argv))
	LB_CALL_MPI(MPI_Barrier(MPI_COMM_WORLD))
#endif /* LB_ENABLE_MPI */

	lblua_main(argc, argv, modules);

#ifdef LB_ENABLE_MPI
	LB_CALL_MPI(MPI_Finalize())
#endif /* LB_ENABLE_MPI */

	return 0;
}
